<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\{ DateUtil, Environment, NovoCodigo, Repositories };

use Ems\Services\{ EstruturaService, EmailService };
use Ems\Repository\{ SolicitacaoRepository, EscalaRepository, OcupacaoRepository, MovimentacaoRepository, ValeTransporteRepository, FeriasRepository, AlteSalarioRepository, MinhaHistoriaRepository };

use Ems\Util\{ ArrayUtil, Service, Utilities, ExpectedException, Utils };

use Ems\Logic\ProcessamentoSolicitacao\{ ProcessamentoSolicitacao, ProcessamentoSolicitacaoFactory };

use Teknisa\Model\Gestaopessoas\Entities\{ GpeFaseaprovsoli, GpeFasetramsoli, GpeSolicitacao };

class SolicitacaoService extends Service {

    /** @var EstruturaService */
    protected $estruturaService;

    /** @var EmailService */
    protected $emailService;

    /** @var Environment */
    protected $environment;

    /** @var SolicitacaoRepository */
    protected $solicitacaoRepository;

    /** @var array */
    protected $solicitacaoInfo;

    /** @var EntityManager */
    protected $entityManager;
    
    const IDAPROVACAOSEC_APROVADO       = 'A';
    const IDAPROVACAOSEC_PENDENTE       = 'P';
    const IDAPROVACAOSEC_REPROVADO      = 'R';
    const IDAPROVACAOSEC_EMANDAMENTO    = 'E';

    public function __construct(
        EntityManager            $entityManager,
        SolicitacaoRepository    $solicitacaoRepository,
        EscalaRepository         $escalaRepository,
        OcupacaoRepository       $ocupacaoRepository,
        MovimentacaoRepository   $movimentacaoRepository,
        ValeTransporteRepository $valeTransporteRepository,
        FeriasRepository         $feriasRepository,
        AlteSalarioRepository    $alteSalarioRepository,
        MinhaHistoriaRepository  $minhaHistoriaRepository,
        Environment              $environment,
        EstruturaService         $estruturaService
    ) {
        $this->entityManager            = $entityManager;
        $this->solicitacaoRepository    = $solicitacaoRepository;
        $this->escalaRepository         = $escalaRepository;
        $this->ocupacaoRepository       = $ocupacaoRepository;
        $this->movimentacaoRepository   = $movimentacaoRepository;
        $this->valeTransporteRepository = $valeTransporteRepository;
        $this->feriasRepository         = $feriasRepository;
        $this->alteSalarioRepository    = $alteSalarioRepository;
        $this->minhaHistoriaRepository  = $minhaHistoriaRepository;
        $this->environment              = $environment;
        $this->estruturaService         = $estruturaService;
    }

    private $labelFieldPessoa = [
        "NRVINCULOM" => "Nr. Vínculo", "NMVINCULOM" => "Nome Vínculo", "NMOCUPACAOH" => "Ocupação (Cargo)", "VRSALARIO" => "Valor Salário"
    ];
    
    public function getEntityRepositoryName(): string {
        return Repositories::GPE_SOLICITACAO;
    }

    public function getEntityKeysName(): array {
        return [
            'NRORG',
            'NRSOLICITACAO'
        ];
    }

    public function getSolicitacoesVinculosGestor(
        int     $nrEstrutura,
        ?string $cdOperadorAprovante = null
    ): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $dtmescompetenc = DateUtil::getPrimeiroDiaDoMes(new \Setforms2\Shared\ORM\DateTime())->format(DateUtil::FORMATO_BRASILEIRO);

        $nrEstruturasString = null;

        if ($nrEstrutura !== -1) {
            $nrEstruturas = $this->estruturaService->
                getTodasEstruturasInferiores($nrEstrutura, true);

            $nrEstruturasString = ArrayUtil::toStringSQL($nrEstruturas);
        }

        $params = [
            'P_NRORG'                   => $nrOrg,
            'P_NRESTRUTURAS'            => $nrEstruturasString,
            'P_DTMESCOMPETENC'          => $dtmescompetenc,
            'P_CDOPERADOR_APROVANTE'    => $cdOperadorAprovante,
        ];

        $solicitacoes = $this->solicitacaoRepository->
            getSolicitacoesVinculosGestor($params);

        foreach ($solicitacoes as &$solicitacao) {
            // Remover quebras de linha e caracteres em branco
            $dsobssolicitacao = preg_replace('/[\n\r]+|\s{2,}/', ' ', $solicitacao['DSOBSSOLICITACAO']);
            if (mb_strlen($dsobssolicitacao, 'UTF-8') > 40) {
                $solicitacao['DSOBSSOLICITACAOABREV'] = mb_substr($dsobssolicitacao, 0, 40, 'UTF-8') . '...';
            } else {
                $solicitacao['DSOBSSOLICITACAOABREV'] = $solicitacao['DSOBSSOLICITACAO'];
            }
        }      

        return $solicitacoes;
    }

    public function getItemSolicitacao($nrsolicitacao) {
        $nrorg = $this->environment->getNrorg();
        return $this->solicitacaoRepository->getItemSolicitacao($nrorg, $nrsolicitacao);
    }

    public function getSolicitacao($nrsolicitacao) {
        $nrorg = $this->environment->getNrorg();
        return $this->solicitacaoRepository->getSolicitacao($nrorg, $nrsolicitacao);
    }

    public function getTiposolicitacao(?int $nrTipoSolicitacao = null, ?int $nrVinculom = null) {
        $nrorg = $this->environment->getNrorg();
        $tipos = $this->solicitacaoRepository->getTiposolicitacao($nrorg, $nrTipoSolicitacao);

        foreach($tipos as &$tipo) {
            if ($tipo['NRTIPOSOLICITACAO'] == '6') {
                $tipo['NMTIPOSOLICITACAO'] = 'Solicitação de Seleção de Pessoal';
            }

            /* Preenchimento das informacoes do relacionamento do vinculo com o modelo de aprovacao */

            if (!empty($nrVinculom)) {
                $relacModeloTipoAprovSoli = $this->entityManager->getRepository(Repositories::GPE_SOLICITACAO)->getRelacionamentoModeloTipoSoli(
                    $nrorg, $tipo['NRTIPOSOLICITACAO'], $nrVinculom, Utils::getParamters()->getDtmescomptencDiaCorrente()
                );

                if (empty($relacModeloTipoAprovSoli)) {
                    continue;
                }

                $tipo['NRPERFACESTIPOSOLI'] = $relacModeloTipoAprovSoli['NRPERFACESTIPOSOLI'] ?? null;
                $tipo['NRMODELOAPROVSOLI'] = $relacModeloTipoAprovSoli['NRMODELOAPROVSOLI'] ?? null;
                $tipo['DSCONSIDERACAO'] = $relacModeloTipoAprovSoli['DSCONSIDERACAO'] ?? null;
            }
        }

        return $tipos;
    }

    public function getTipoRelacionamento() {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tiposRel = $this->solicitacaoRepository->getTipoRelacionamento(
            $nrOrg, $nrOrgPadrao
        );

        return $tiposRel;
    }

    public function getLinhaTransporteAtiva(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('1/m/Y');

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_DTMESCOMPETENC'  => $dtMesCompetencia,
        ];

        $linhas = $this->solicitacaoRepository->getLinhaTransporteAtiva(
            $params
        );

        return $linhas;
    }

    public function getTipolinhavinc(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tiposLinhas = $this->solicitacaoRepository->getTipolinhavinc(
            $nrOrg, $nrOrgPadrao
        );

        return $tiposLinhas;
    }

    public function getIntegtransp(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('01/m/Y');

        $params = [
            'P_NRORG'               => $nrOrg,
            'P_NRORG_PADRAO'        => $nrOrgPadrao,
            'P_DTMESCOMPETENC'      => $dtMesCompetencia,
        ];

        $numerosIntegracao = $this->solicitacaoRepository->getIntegtransp(
            $params
        );

        return $numerosIntegracao;
    }

    public function getSentidolinha(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $numerosIntegracao = $this->solicitacaoRepository->getSentidolinha(
            $nrOrg, $nrOrgPadrao
        );

        return $numerosIntegracao;
    }

    public function getBeneficio(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $beneficios = $this->solicitacaoRepository->getBeneficio(
            $nrOrg, $nrOrgPadrao
        );

        return $beneficios;
    }

    public function getPessoaRelacionamentos(
        int $nrPessoa
    ): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('01/m/Y');

        $params = [
            'P_NRORG'               => $nrOrg,
            'P_NRORGPADRAO'         => $nrOrgPadrao,
            'P_DTMESCOMPETENC'      => $dtMesCompetencia,
            'P_NRPESSOA'            => $nrPessoa,
            'P_NRTIPORELACIONA'     => null,
        ];

        $pessoas = $this->solicitacaoRepository->getPessoaRelacionamentos(
            $params
        );

        return $pessoas;
    }

    public function getTipoDependente(): array {
        $nrOrgPadrao = $this->environment->getNrOrgPadrao();

        $tiposDep = $this->solicitacaoRepository->getTipoDependente($nrOrgPadrao);

        return $tiposDep;
    }

    public function getTemplate(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $templates = $this->solicitacaoRepository->getTemplate($nrOrg);

        return $templates;
    }

    public function getTipoprocessos(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $tipos = $this->solicitacaoRepository->getTipoprocessos($nrOrg);

        return $tipos;
    }

    public function getParcnegocioVinculos(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $tipos = $this->solicitacaoRepository->getParcnegocioVinculos($nrOrg);

        return $tipos;
    }

    public function getTipoTransferencia(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tipos = $this->solicitacaoRepository->getTipoTransferencia(
            $nrOrg, $nrOrgPadrao
        );

        return $tipos;
    }

    public function getMovimentacaoTransferencia(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('01/m/Y');

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRORG_PADRAO' => $nrOrgPadrao,
            'P_DTMESCOMPETENC' => $dtMesCompetencia,
        ];

        $tipos = $this->solicitacaoRepository->getMovimentacaoTransferencia(
            $params
        );

        return $tipos;
    }

    public function getMotivoTransferencia(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $motivos = $this->solicitacaoRepository->getMotivoTransferencia(
            $nrOrg
        );

        return $motivos;
    }

    public function getFasesSolicitacao(
        string $nrSolicitacao
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $fases = $this->solicitacaoRepository->getFasesSolicitacao(
            $nrOrg, $nrSolicitacao
        );

        return $fases;
    }
    
    public function aprovaSolicitacao(
        string  $nrSolicitacao,
        string  $cdOperadorAprovante,
        ?string $dsObservacao           = null,
        ?array  $extraRow               = [],
        ?bool   $isVariasSolicitacoes   = false
    ): bool {
        return $this->atualizaSolicitacao(
            $nrSolicitacao,
            self::IDAPROVACAOSEC_APROVADO,
            $cdOperadorAprovante,
            $dsObservacao,
            $extraRow,
            $isVariasSolicitacoes
        );
    }
    
    public function reprovaSolicitacao(
        string  $nrSolicitacao,
        string  $cdOperadorAprovante,
        ?string $dsObservacao           = null,
        ?array  $extraRow               = []
    ): bool {
        return $this->atualizaSolicitacao(
            $nrSolicitacao,
            self::IDAPROVACAOSEC_REPROVADO,
            $cdOperadorAprovante,
            $dsObservacao,
            $extraRow
        );
    }
    
    /**
     * Atualiza o status e as fases de aprovacao de solicitacao. Caso a ultima fase for aprovada procede com as insercoes de acordo com as entidades ProcessmentoSolicitacao.
     * 
     * @param string  $nrSolicitacao
     * @param string  $idAprovacao
     * @param string  $cdOperadorAprovante
     * @param ?string $dsObservacao
     * @param ?array  $extraRow
     * 
     * @return bool
     */
    public function atualizaSolicitacao(
        string  $nrSolicitacao,
        string  $idAprovacao,
        string  $cdOperadorAprovante,
        ?string $dsObservacao           = null,
        ?array  $extraRow               = [],
        ?bool   $isVariasSolicitacoes   = false
    ): bool {
        $solicitacao = $this->entityManager->getRepository(Repositories::GPE_SOLICITACAO)->findOneBy(array('nrorg' => $this->environment->getNrOrg(), 'nrsolicitacao' => $nrSolicitacao));
        if (empty($solicitacao)) throw new \Exception('Solicitação não encontrada.');
        
        $this->setSolicitacaoInfo($solicitacao->getNrsolicitacao(), $isVariasSolicitacoes);
        
        /* Checa se a solicitacao esta em um estado que permite a aprovacao */

        if (!$this->isSolicitacaoAtualValidaAprovacao()) {
            throw new ExpectedException('Não foi possível aprovar. A solicitação já está aprovada/rejeitada');
        }

        /* Checa se o operador que esta realizando a aprovacao tem acesso */

        if (!$this->checkPerfilAprovacaoSolicitacaoAtual($cdOperadorAprovante)) {
            throw new ExpectedException('Operador não possui perfil de acesso para aprovar essa fase da solicitação. Reveja as informações do modelo de fases da aprovação solicitação.');
        }

        /* Invoca as rotinas de aprovacao se a solicitacao esta na ultima fase */

        if ($this->isSolicitacaoAtualEmUltimaEtapa() && $idAprovacao === self::IDAPROVACAOSEC_APROVADO) {
            $processamentoSolicitacao = $this->getProcessamentoSolicitacao($solicitacao, $extraRow);
            if (!empty($processamentoSolicitacao)) {
                $processamentoSolicitacao->efetivar();
            }
        }

        if($idAprovacao === self::IDAPROVACAOSEC_REPROVADO) {
            $processamentoSolicitacao = $this->getProcessamentoSolicitacao($solicitacao, $extraRow);
            $processamentoSolicitacao->rejeitar();
        }

        /* Atualiza a informacoes da solicitacao e de tramitacao */

        $idaprovacao = $this->aprovaFaseTramitacao($solicitacao->getNrsolicitacao(), $solicitacao->getNrtiposolicitacao(), $idAprovacao, $cdOperadorAprovante, $dsObservacao);

        $this->update(
            [
                'NRORG'             => $this->environment->getNrorg(),
                'NRSOLICITACAO'     => $nrSolicitacao,
                'IDAPROVACAOSEC'    => $idaprovacao,
                'CDOPERULTATU'      => $cdOperadorAprovante,
            ]
        );
        
        return true;
    }
    
    /**
     * Valida a existencia de um modelo para um determinado tipo de solicitacao.
     * 
     * @param $nrtiposolicitacao
     * 
     * @return array
     */
    public function validaModeloAprovacao($nrtiposolicitacao) {
        $erro["Bloquear"] = false;
        
        $perfil = $this->validaPerfildeAcesso($nrtiposolicitacao);
        if(!$perfil) {
            $erro["Texto"] = "Não existe perfil de acesso para este tipo de solicitação.";
            $erro["Bloquear"] = true;
        } else {
            if(!$perfil->getNrmodeloaprovsoli()) {
                $erro["Texto"] = "Modelo de aprovação não parametrizado para este tipo de solicitação.";
                $erro["Bloquear"] = true;
            }
        }

        return $erro;
    }
    
    /**
     * Busca o registro de GPE_PERFACESTIPOSOLI associado ao tipo de solicitacao.
     * 
     * @param $tiposoli
     * 
     * @return ?GpePerfacestiposoli
     */
    public function validaPerfildeAcesso($tiposoli){
        $perfil = $this->entityManager->getRepository(Repositories::GPE_PERFACESTIPOSOLI)->findOneBy(array('nrorg'=>$this->environment->getNrorg(),'nrtiposolicitacao'=>$tiposoli));
        if (!$perfil) {
            return false;
        }
        return $perfil;
    }
    
    /**
     * Retorna o proximo IDAPROVACAOSEC a ser atualizado na solicitacao. Caso nao existam proximas fases, null eh retornado.
     * 
     * @param string  $nrSolicitacao
     * @param string  $nrTipoSolicitacao
     * @param string  $idAprovacao
     * @param string  $cdOperadorAprovante
     * @param ?string $dsObservacao
     * 
     * @return ?string
     */
    public function aprovaFaseTramitacao(
        string  $nrSolicitacao,
        string  $nrTipoSolicitacao,
        string  $idAprovacao,
        string  $cdOperadorAprovante,
        ?string $dsObservacao = null
    ): ?string {

        $faseatual = $this->entityManager->getRepository(Repositories::GPE_FASETRAMSOLI)->retornaFaseatual($this->environment->getNrorg(), $nrSolicitacao, Repositories::class);

        if ($faseatual) {
            $nrordem = $faseatual->getNrordem();
        } else {
            $nrordem = 0;
        }

        $nrFaseSeguinte = $this->solicitacaoInfo['NRFASEAPROVSOLI_SEGUINTE'];
        $proximafase = $this->entityManager->getRepository(Repositories::GPE_FASEAPROVSOLI)->findOneBy(['nrorg' => $this->environment->getNrorg(), 'nrfaseaprovsoli' => $nrFaseSeguinte]);
        $naoExisteProximaFase = empty($proximafase);
        if ($naoExisteProximaFase) {
            return null;
        }

        if ($idAprovacao == 'A') {
            $idaprovado = 'S';
        } else {
            $idaprovado = 'N';
        }

        $fasetram = $this->insereFaseTramSoli($nrSolicitacao, $proximafase->getNrfaseaprovsoli(), $idaprovado, $cdOperadorAprovante, $dsObservacao);

        $ultimafase = $this->entityManager->getRepository(Repositories::GPE_FASEAPROVSOLI)->findOneBy(
            array('nrorg' => $this->environment->getNrorg(), 'nrmodeloaprovsoli' => $proximafase->getNrmodeloaprovsoli()),
            array('nrordem' => 'DESC')
        );

        /*
        Se for aprovar e a fase for a última do modelo, muda o status da solicitação para aprovada
        Se for aprovar e não for a última fase o status da solicitação fica como em andamento
        Se for reprovar e a fase for obrigatória muda o status da solicitação para reprovada
        Se for reprovar e a fase não for obrigatória o status da solicitação fica como em andamento
        */
        $novoIdAprovacao = null;
        
        if ($idAprovacao == 'A') {
            if ($proximafase->getNrfaseaprovsoli() == $ultimafase->getNrfaseaprovsoli()) {
                $novoIdAprovacao = 'A';
            } else {
                $novoIdAprovacao = 'E';
            }
        } else {
            if ($proximafase->getIdfaseobrigatoria() == 'S') {
                $novoIdAprovacao = 'R';
            } else {
                $novoIdAprovacao = 'E';
            }
        }

        return $novoIdAprovacao;
    }

    /**
     * Verifica se o operador aprovante possui perfil de acesso adequado para a fase de aprovacao da solicitacao
     * @param string $nrSolicitacao
     * @param string $cdOperadorAprovante
     * @return bool
     */
    public function checkPerfilAprovacaoSolicitacaoAtual(
        string $cdOperadorAprovante
    ): bool {
        $nrSolicitacao = $this->solicitacaoInfo['NRSOLICITACAO'];

        $solicitacao = $this->entityManager->getRepository(Repositories::GPE_SOLICITACAO)->findOneBy(['nrorg' => $this->environment->getNrorg(), 'nrsolicitacao' => $nrSolicitacao]);
        if (empty($solicitacao)) {
            return false;
        }

        $nrFaseSeguinte = $this->solicitacaoInfo['NRFASEAPROVSOLI_SEGUINTE'];
        $proximafase = $this->entityManager->getRepository(Repositories::GPE_FASEAPROVSOLI)->findOneBy(['nrorg' => $this->environment->getNrorg(), 'nrfaseaprovsoli' => $nrFaseSeguinte]);
        $naoExisteProximaFase = empty($proximafase);
        if ($naoExisteProximaFase) {
            return null;
        }
        
        $perfilfase = explode(',',$proximafase->getDsnrperfilacesso());

        $operador = $this->entityManager->getRepository(Repositories::OPERADOR)->findOneBy(['cdoperador' => $cdOperadorAprovante]);

        $perfiloper = $this->entityManager->getRepository(Repositories::PERFACESSPARC)->findBy(
            ['nrorg' => $this->environment->getNrorg(), 'nrparcnegocio' => $operador->getNrparcnegocio(), 'nrperfilacesso' => $perfilfase]
        );

        return !empty($perfiloper);
    }
    
    /**
     * Inclui o registro de uma fase para uma solicitacao.
     * 
     * @param string  $nrSolicitacao,
     * @param string  $nrFaseAprovSoli,
     * @param string  $idAprovado,
     * @param string  $cdOperadorAprovante,
     * @param ?string $dsObservacao = null
     * 
     * @return GpeFasetramsoli
     */
    public function insereFaseTramSoli(
        string  $nrSolicitacao,
        string  $nrFaseAprovSoli,
        string  $idAprovado,
        string  $cdOperadorAprovante,
        ?string $dsObservacao = null
    ): GpeFasetramsoli {

        /** @var \Teknisa\Model\Gestaopessoas\Entities\GpeFasetramsoli $gpe_fasetramsoli */
        $gpe_fasetramsoli = Repositories::GPE_FASETRAMSOLI;    
        
        $faseTram = new $gpe_fasetramsoli();

        $faseTram->setNrsolicitacao($nrSolicitacao);
        $faseTram->setNrfaseaprovsoli($nrFaseAprovSoli);
        $faseTram->setIdaprovado($idAprovado);
        $faseTram->setDsobservacao($dsObservacao);
        $faseTram->setNrorg($this->environment->getNrorg());
        $faseTram->setDtinclusao(DateUtil::getDataAtual());
        $faseTram->setCdoperinclusao($cdOperadorAprovante);
        $faseTram->setNrorginclusao($this->environment->getNrorg());
        
        $nrFaseTramSoli = NovoCodigo::geraCodigo($this->environment->getNrorg(), 'GPE_FASETRAMSOLI', 1, 12, true);
        $faseTram->setNrfasetramsoli($nrFaseTramSoli);
        
        $this->entityManager->persist($faseTram);
        $this->entityManager->flush();
        
        return $faseTram;
    }
    
    /**
     * Busca a entidade de processamento da aprovacao de uma solicitacao capazes de efetiva-las.
     * 
     * @param GpeSolicitacao  $solicitacao,
     * @param ?array          $extraRow
     * 
     * @return ?ProcessamentoSolicitacao
     */
    public function getProcessamentoSolicitacao(
        GpeSolicitacao  $solicitacao,
        ?array          $extraRow
    ): ?ProcessamentoSolicitacao {
        return ProcessamentoSolicitacaoFactory::createProcessamentoSolicitacao($solicitacao, $extraRow);
    }
    
    public static function isTipoSoliOnlyEmail(string $tipoSoli): bool {
        return $tipoSoli < 1;
    }

    /**
     * Verifica se a solicitacao atual esta em sua ultima etapa
     * @return bool
     */
    public function isSolicitacaoAtualEmUltimaEtapa(): bool {
        return !empty($this->solicitacaoInfo)
            ? $this->solicitacaoInfo['FASE_ATUAL_EH_ULTIMA'] == 'S'
            : false;
    }

    /**
     * Verifica se a solicitacao atual a ser analisada eh valida ou nao
     * @return bool
     */
    public function isSolicitacaoAtualValidaAprovacao(): bool {
        return !empty($this->solicitacaoInfo)
            ? $this->solicitacaoInfo['FASE_ATUAL_VALIDA_APROVACAO'] == 'S'
            : false;
    }

    public function getInfoFase(int $nrSolicitacao): array {
        $infoFase = $this->entityManager->getRepository(Repositories::GPE_SOLICITACAO)->getInfoFase(
            $this->environment->getNrorg(), $nrSolicitacao
        );
        return empty($infoFase) ? [] : $infoFase;
    }

    /**
     * Responsavel por definir as informacoes e status das ocorrencias
     * 
     * @param int $nrSolicitacao
     * @return bool
     */
    private function setSolicitacaoInfo(
        int $nrSolicitacao,
        bool $isVariasSolicitacoes = false
    ): void {
        if($isVariasSolicitacoes) {
            $this->solicitacaoInfo = array();
        }
        if (!empty($this->solicitacaoInfo)) {
            return;
        }
        $this->solicitacaoInfo = $this->entityManager->getRepository(Repositories::GPE_SOLICITACAO)->getInfoFase(
            $this->environment->getNrorg(), $nrSolicitacao
        );
    }
    
    public function getAlteracaoEscalaTrabalho(
        int $nrVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $escalaTrabalho = $this->escalaRepository->getEscalaTrabalhoVinculo($nrOrg, $nrVinculo);
        $nrEscalaTraBM = $escalaTrabalho["NRESCALATRABM"] ?? 0;

        $turno = $this->escalaRepository->getTurnoVinculo($nrOrg, $nrVinculo, $nrEscalaTraBM);
    
        $result = [
            'ESCALATRABALHO' => $escalaTrabalho,
            'TURNO' => $turno
        ];

        return $result;
    }

    public function getAlteracaoOcupacao(
        int $nrVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $dtmescompetenc = $this->environment->getDtmescomptencDiaCorrente();
        $ocupacao = $this->ocupacaoRepository->getOcupacaoVinculo($nrOrg, $dtmescompetenc, $nrVinculo);

        return $ocupacao;
    }

    public function getAlteracaoTransferencia(
        int $nrVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $dtmescompetenc = $this->environment->getDtmescomptencDiaCorrente();
        $movimentacao = $this->movimentacaoRepository->getUltimaMovimentacao($nrOrg, $dtmescompetenc, $nrVinculo);

        $result = $this->getAlteracaoEscalaTrabalho($nrVinculo);

        $result['MOVIMENTACAO'] = $movimentacao;

        return $result;
    }

    public function getLinhasVinculosVT(
        int $nrVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $dtAtual = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);

        $linhas = $this->valeTransporteRepository->getLinhasVinculoVT($nrOrg, $nrVinculo, $dtAtual);

        $tarifas = $this->valeTransporteRepository->getTarifasVT($dtAtual);

            
        $result = [
            'LINHAS' => $linhas,
            'TARIFAS' => $tarifas
        ];

        return $result;
    }

    public function getDadosFeriasVinculo(
        int $nrVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $programacaoFerias = $this->feriasRepository->getPeriodosFeriasVinculo($nrOrg, $nrVinculo);

        return $programacaoFerias;
    }

    public function getSolicitacaoSubstituicaoOrVinculoSubst(
        int $nrVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrTipoSolicitacao = ProcessamentoSolicitacaoFactory::TP_SOLI_SUBSTITUICAO;

        $solicitacoes = $this->solicitacaoRepository->getSolicitacaoByTipoEVinculo($nrOrg, $nrVinculo, $nrTipoSolicitacao);
        $vinculo = $this->solicitacaoRepository->getVinculoSubstituido($nrOrg, $nrVinculo);

        $result = [
            'SOLICITACOES' => $solicitacoes,
            'VINCULO' => $vinculo
        ];

        return $result;
    }

    public function setDataSoliSubstituicaoVinculo(
        int $nrVinculo,
        int $nrSolicitacao
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();
        $dtmescompetenc = $this->environment->getDtmescomptencDiaCorrente();
        $cdoperador = Utils::getParamters()->getCdoperador();

        $ocupacao = $this->ocupacaoRepository->getOcupacaoVinculo($nrOrg, $dtmescompetenc, $nrVinculo);
        $ultimoSalario = $this->alteSalarioRepository->getUltimoSalario($nrOrg, $dtmescompetenc, $nrVinculo);
        $vinculo = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array('nrorg' => $nrOrg, 'nrvinculom' => $nrVinculo));

        $itens = [
            "NRVINCULOM"  => $nrVinculo,
            "NMVINCULOM"  => $vinculo->getNmvinculom(),
            "NMOCUPACAOH" => $ocupacao[0]["NMOCUPACAOH"],
            "VRSALARIO"   => $ultimoSalario[0]["VRSALARIO"]
        ];

        foreach ($itens as $key => $field) {
            if(!empty($field)){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrOrg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrSolicitacao, $key, $this->labelFieldPessoa[$key], strval($field), $nrOrg, $cdoperador);
            }
        }

    }
    
}
